#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file


"""Test functions with variable args"""


def plain_list_dict_args_function(plain, *arg_list, **arg_dict):
    print("plain", plain, "arg_list", arg_list, "arg_dict", arg_dict)


def plain_list_args_function(plain, *arg_list):
    print(plain, arg_list)


def plain_dict_args_function(plain, **arg_dict):
    print(plain, arg_dict)


print("Function with plain arg and varargs dict:")
plain_dict_args_function(1, a=2, b=3, c=4)
plain_dict_args_function(1)

print("Function with plain arg and varargs list:")
plain_list_args_function(1, 2, 3, 4)
plain_list_args_function(1)

print("Function with plain arg, varargs list and varargs dict:")
plain_list_dict_args_function(1, 2, z=3)
plain_list_dict_args_function(1, 2, 3)
plain_list_dict_args_function(1, a=2, b=3, c=4)


def list_dict_args_function(*arg_list, **arg_dict):
    print(arg_list, arg_dict)


def list_args_function(*arg_list):
    print(arg_list)


def dict_args_function(**arg_dict):
    print(arg_dict)


print("Function with plain arg and varargs dict:")
dict_args_function(a=2, b=3, c=4)
dict_args_function()

print("Function with plain arg and varargs list:")
list_args_function(2, 3, 4)
list_args_function()

print("Function with plain arg, varargs list and varargs dict:")
list_dict_args_function(2, z=3)
list_dict_args_function(2, 3)
list_dict_args_function(a=2, b=3, c=4)

#     Python tests originally created or extracted from other peoples work. The
#     parts were too small to be protected.
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.



#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file


""" Progress bars in Nuitka.

This is responsible for wrapping the rendering of progress bar and emitting tracing
to the user while it's being displayed.

"""

from contextlib import contextmanager

from nuitka import Tracing
from nuitka.Tracing import general
from nuitka.utils.Importing import importFromInlineCopy
from nuitka.utils.ThreadedExecutor import RLock
from nuitka.utils.Utils import isWin32Windows

# spell-checker: ignore tqdm,ncols

# Late import and optional to be there.
use_progress_bar = False
tqdm = None
colorama = None


class NuitkaProgressBar(object):
    def __init__(self, iterable, stage, total, min_total, unit):
        self.total = total

        # The minimum may not be provided, then default to 0.
        self.min_total = min_total

        # No item under work yet.
        self.item = None

        # No progress yet.
        self.progress = 0

        # Render immediately with 0 progress, and setting disable=None enables tty detection.
        self.tqdm = tqdm(
            iterable=iterable,
            initial=self.progress,
            total=max(self.total, self.min_total)
            if self.min_total is not None
            else None,
            unit=unit,
            disable=None,
            leave=False,
            dynamic_ncols=True,
            bar_format="{desc}{percentage:3.1f}%|{bar:25}| {n_fmt}/{total_fmt}{postfix}",
        )

        self.tqdm.set_description(stage)
        self.setCurrent(self.item)

    def __iter__(self):
        return iter(self.tqdm)

    def updateTotal(self, total):
        if total != self.total:
            self.total = total
            self.tqdm.total = max(total, self.min_total)

    def setCurrent(self, item):
        if item != self.item:
            self.item = item

            if item is not None:
                self.tqdm.set_postfix_str(item)
            else:
                self.tqdm.set_postfix()

    def update(self):
        self.progress += 1
        self.tqdm.update(1)

    def clear(self):
        self.tqdm.clear()

    def close(self):
        self.tqdm.close()

    @contextmanager
    def withExternalWritingPause(self):
        with self.tqdm.external_write_mode():
            yield


def _getTqdmModule():
    global tqdm  # singleton, pylint: disable=global-statement

    if tqdm:
        return tqdm
    elif tqdm is False:
        return None
    else:
        tqdm = importFromInlineCopy("tqdm", must_exist=False, delete_module=True)

        if tqdm is None:
            try:
                # Cannot use import tqdm due to pylint bug.
                import tqdm as tqdm_installed  # pylint: disable=I0021,import-error

                tqdm = tqdm_installed
            except ImportError:
                # We handle the case without inline copy too, but it may be removed, e.g. on
                # Debian it's only a recommended install, and not included that way.
                pass

        if tqdm is None:
            tqdm = False
            return None

        tqdm = tqdm.tqdm

        # Tolerate the absence ignore the progress bar
        tqdm.set_lock(RLock())

        return tqdm


def enableProgressBar():
    global use_progress_bar  # singleton, pylint: disable=global-statement
    global colorama  # singleton, pylint: disable=global-statement

    if _getTqdmModule() is not None:
        use_progress_bar = True

        if isWin32Windows():
            if colorama is None:
                colorama = importFromInlineCopy(
                    "colorama", must_exist=True, delete_module=True
                )

            colorama.init()


def setupProgressBar(stage, unit, total, min_total=0):
    # Make sure the other was closed.
    assert Tracing.progress is None

    if use_progress_bar:
        Tracing.progress = NuitkaProgressBar(
            iterable=None,
            stage=stage,
            total=total,
            min_total=min_total,
            unit=unit,
        )


def reportProgressBar(item, total=None, update=True):
    if Tracing.progress is not None:
        try:
            if total is not None:
                Tracing.progress.updateTotal(total)

            Tracing.progress.setCurrent(item)

            if update:
                Tracing.progress.update()
        except Exception as e:  # Catch all the things, pylint: disable=broad-except
            # We disable the progress bar now, because it's causing issues.
            general.warning("Progress bar disabled due to bug: %s" % (str(e)))
            closeProgressBar()


def closeProgressBar():
    """Close the active progress bar.

    Returns: int or None - if displayed, the total used last time.
    """

    if Tracing.progress is not None:
        # Retrieve that previous total, for repeated progress bars, it
        # can be used as a new minimum.
        result = Tracing.progress.total

        Tracing.progress.close()
        Tracing.progress = None

        return result


def wrapWithProgressBar(iterable, stage, unit):
    if tqdm is None:
        return iterable
    else:
        result = NuitkaProgressBar(
            iterable=iterable, unit=unit, stage=stage, total=None, min_total=None
        )

        Tracing.progress = result

        return result


@contextmanager
def withNuitkaDownloadProgressBar(*args, **kwargs):
    if not use_progress_bar or _getTqdmModule() is None:
        yield
    else:

        class NuitkaDownloadProgressBar(tqdm):
            # spell-checker: ignore bsize, tsize
            def onProgress(self, b=1, bsize=1, tsize=None):
                if tsize is not None:
                    self.total = tsize
                self.update(b * bsize - self.n)

        kwargs.update(
            disable=None,
            leave=False,
            dynamic_ncols=True,
            bar_format="{desc} {percentage:3.1f}%|{bar:25}| {n_fmt}/{total_fmt}{postfix}",
        )

        with NuitkaDownloadProgressBar(*args, **kwargs) as progress_bar:
            yield progress_bar.onProgress


#     Part of "Nuitka", an optimizing Python compiler that is compatible and
#     integrates with CPython, but also works on its own.
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.



#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file


from __future__ import print_function


def separator():
    print("*" * 80)


def dictOrderCheck():
    def key1():
        print("key1 called")

        return 1

    def key2():
        print("key2 called")

        return 2

    def value1():
        print("value1 called")

        return 11

    def value2():
        print("value2 called")

        return 22

    print("Checking order of calls in dictionary creation from callables:")

    print({key1(): value1(), key2(): value2()})

    try:
        (1 / 0)[1.0j / 0] = 1.0 / 0
    except ZeroDivisionError as e:
        print("Expected exception caught:", repr(e))
    try:
        (1 / 0)[1.0 / 0] = 1
    except ZeroDivisionError as e:
        print("Expected exception caught:", repr(e))
    try:
        (1 / 0)[1] = 1.0 / 0
    except ZeroDivisionError as e:
        print("Expected exception caught:", repr(e))


def listOrderCheck():
    def value1():
        print("value1 called")

        return 11

    def value2():
        print("value2 called")

        return 22

    print([value1(), value2()])


def sliceOrderCheck():
    print("Slices:")
    d = list(range(10))

    def lvalue():
        print("lvalue", end=" ")

        return d

    def rvalue():
        print("rvalue", end=" ")

        return range(2)

    def rvalue4():
        print("rvalue", end=" ")

        return range(4)

    def low():
        print("low", end=" ")

        return 0

    def high():
        print("high", end=" ")

        return 4

    def step():
        print("step", end=" ")

        return 2

    print("Complex slice lookup:", end=" ")
    print(lvalue()[low() : high() : step()])

    print("Complex slice assignment:", end=" ")
    lvalue()[low() : high() : step()] = rvalue()
    print(d)

    print("Complex slice del:", end=" ")
    del lvalue()[low() : high() : step()]
    print(d)

    print("Complex inplace slice operation", end=" ")
    # TODO: This gives an error in CPython, but not in Nuitka.
    # lvalue()[ low() : high() : step() ] += rvalue()
    print(d)

    d = list(range(10))

    print("Simple slice lookup", end=" ")
    print(lvalue()[low() : high()])

    print("Simple slice assignment", end=" ")
    lvalue()[3 + low() : 3 + high()] = rvalue()
    print(d)

    print("Simple slice del", end=" ")
    del lvalue()[3 + low() : 3 + high()]
    print(d)

    print("Simple inplace slice operation", end=" ")
    lvalue()[low() : high()] += rvalue4()
    print(d)


def subscriptOrderCheck():
    print("Subscripts:")
    d = {}

    def lvalue():
        print("lvalue", end=" ")

        return d

    def rvalue():
        print("rvalue", end=" ")

        return 2

    def subscript():
        print("subscript", end=" ")

        return 1

    print("Assigning subscript:")
    lvalue()[subscript()] = rvalue()
    print(d)

    print("Lookup subscript:")
    print(lvalue()[subscript()])

    print("Deleting subscript:")
    del lvalue()[subscript()]
    print(d)


def attributeOrderCheck():
    def lvalue():
        print("lvalue", end=" ")

        return lvalue

    def rvalue():
        print("rvalue", end=" ")

        return 2

    print("Attribute assignment order:")

    lvalue().xxx = rvalue()
    print("Assigned was indeed:", lvalue.xxx)

    print("Checking attribute assignment to unassigned value from unassigned:")
    try:
        undefined_global_zzz.xxx = undefined_global_yyy
    except Exception as e:
        print("Expected exception caught:", repr(e))
    else:
        assert False

    try:
        (1 / 0).x = 1.0 / 0
    except ZeroDivisionError as e:
        print("Expected exception caught:", repr(e))


def compareOrderCheck():
    def lvalue():
        print("lvalue", end=" ")

        return 1

    def rvalue():
        print("rvalue", end=" ")

        return 2

    print("Comparisons:")
    print("==", lvalue() == rvalue())
    print("<=", lvalue() <= rvalue())
    print(">=", lvalue() >= rvalue())
    print("!=", lvalue() != rvalue())
    print(">", lvalue() > rvalue())
    print("<", lvalue() < rvalue())

    print("Comparison used in bool context:")
    print("==", "yes" if lvalue() == rvalue() else "no")
    print("<=", "yes" if lvalue() <= rvalue() else "no")
    print(">=", "yes" if lvalue() >= rvalue() else "no")
    print("!=", "yes" if lvalue() != rvalue() else "no")
    print(">", "yes" if lvalue() > rvalue() else "no")
    print("<", "yes" if lvalue() < rvalue() else "no")


def operatorOrderCheck():
    def left():
        print("left operand", end=" ")

        return 1

    def middle():
        print("middle operand", end=" ")

        return 3

    def right():
        print("right operand", end=" ")

        return 2

    print("Operations:")
    print("+", left() + middle() + right())
    print("-", left() - middle() - right())
    print("*", left() * middle() * right())
    print("/", left() / middle() / right())
    print("%", left() % middle() % right())
    print("**", left() ** middle() ** right())


def generatorOrderCheck():
    print("Generators:")

    def default1():
        print("default1", end=" ")

        return 1

    def default2():
        print("default2", end=" ")

        return 2

    def default3():
        print("default3", end=" ")

        return 3

    def value(x):
        print("value", x, end=" ")
        return x

    def generator(a=default1(), b=default2(), c=default3()):
        print("generator entry")
        yield value(a)
        yield value(b)
        yield value(c)
        print("generator exit")

    result = list(generator())
    print("Result", result)


def classOrderCheck():
    print("Checking order of class constructions:")

    class B1:
        pass

    class B2:
        pass

    def base1():
        print("base1", end=" ")

        return B1

    def base2():
        print("base2", end=" ")

        return B2

    def deco1(cls):
        print("deco1", end=" ")

        return cls

    def deco2(cls):
        print("deco2")

        return B2

    @deco2
    @deco1
    class X(base1(), base2()):
        print("class body", end=" ")

    print


def inOrderCheck():
    print("Checking order of in operator:")

    def container():
        print("container", end=" ")

        return [3]

    def searched():
        print("searched", end=" ")

        return 3

    print("in:", searched() in container())
    print("not in:", searched() not in container())


def unpackOrderCheck():
    class TraceRelease:
        def __init__(self, value):
            self.value = value

        def __del__(self):
            print("Deleting iteration value", self.value)
            pass

    print("Unpacking values:")

    class Iterable:
        def __init__(self):
            self.consumed = 2

        def __iter__(self):
            return Iterable()

        def __del__(self):
            print("Deleted iterable with", self.consumed)

        def next(self):
            print("Next with state", self.consumed)

            if self.consumed:
                self.consumed -= 1
            else:
                raise StopIteration

            return TraceRelease(self.consumed)

        __next__ = next

    iterable = Iterable()

    class RejectAttributeOwnership:
        def __setattr__(self, key, value):
            print("Setting", key, value.value)

    try:
        RejectAttributeOwnership().x, RejectAttributeOwnership().y = a, b = iterable
    except Exception as e:
        print("Caught", repr(e))

    return a, b


def superOrderCheck():
    print("Built-in super:")
    try:
        super(zzz, xxx)
    except Exception as e:
        print("Expected exception caught super 2", repr(e))


def isinstanceOrderCheck():
    print("Built-in isinstance:")
    try:
        isinstance(zzz, xxx)
    except Exception as e:
        print("Expected exception caught isinstance 2", repr(e))


def rangeOrderCheck():
    print("Built-in range:")
    try:
        range(zzz, yyy, xxx)
    except Exception as e:
        print("Expected exception caught range 3", repr(e))

    try:
        range(zzz, xxx)
    except Exception as e:
        print("Expected exception caught range 2", repr(e))


def importOrderCheck():
    print("Built-in __import__:")

    def name():
        print("name", end=" ")

    def globals():
        print("globals", end=" ")

    def locals():
        print("locals", end=" ")

    def fromlist():
        print("fromlist", end=" ")

    def level():
        print("level")

    try:
        __import__(name(), globals(), locals(), fromlist(), level())
    except Exception as e:
        print("Expected exception caught __import__ 5", repr(e))


def hasattrOrderCheck():
    print("Built-in hasattr:")
    try:
        hasattr(zzz, yyy)
    except Exception as e:
        print("Expected exception caught hasattr", repr(e))


def getattrOrderCheck():
    print("Built-in getattr:")
    try:
        getattr(zzz, yyy)
    except Exception as e:
        print("Expected exception caught getattr 2", repr(e))

    try:
        getattr(zzz, yyy, xxx)
    except Exception as e:
        print("Expected exception caught getattr 3", repr(e))

    def default():
        print("default used")

    print("Default side effects:", end=" ")
    print(getattr(1, "real", default()))


def typeOrderCheck():
    print("Built-in type:")
    try:
        type(zzz, yyy, xxx)
    except Exception as e:
        print("Expected exception caught type 3", repr(e))


def iterOrderCheck():
    print("Built-in iter:")
    try:
        iter(zzz, xxx)
    except Exception as e:
        print("Expected exception caught iter 2", repr(e))


def openOrderCheck():
    print("Built-in open:")
    try:
        open(zzz, yyy, xxx)
    except Exception as e:
        print("Expected exception caught open 3", repr(e))


def unicodeOrderCheck():
    print("Built-in unicode:")
    try:
        unicode(zzz, yyy, xxx)
    except Exception as e:
        print("Expected exception caught unicode", repr(e))


def longOrderCheck():
    print("Built-in long:")
    try:
        long(zzz, xxx)
    except Exception as e:
        print("Expected exception caught long 2", repr(e))


def intOrderCheck():
    print("Built-in int:")
    try:
        int(zzz, xxx)
    except Exception as e:
        print("Expected exception caught int", repr(e))


def nextOrderCheck():
    print("Built-in next:")
    try:
        next(zzz, xxx)
    except Exception as e:
        print("Expected exception caught next 2", repr(e))


def callOrderCheck():
    print("Checking nested call arguments:")

    class A:
        def __del__(self):
            print("Doing del inner object")

    def check(obj):
        print("Outer function")

    def p(obj):
        print("Inner function")

    check(p(A()))


def boolOrderCheck():
    print("Checking order of or/and arguments:")

    class A(int):
        def __init__(self, value):
            self.value = value

        def __del__(self):
            print("Doing del of %s" % self)

        def __bool__(self):
            print("Test of %s" % self)
            return self.value != 0

        __nonzero__ = __bool__

        def __str__(self):
            return "<%s %r>" % (self.__class__.__name__, self.value)

    class B(A):
        pass

    class C(A):
        pass

    print("Two arguments, A or B:")
    for a in range(2):
        for b in range(2):
            print("Case %d or %d" % (a, b))
            r = A(a) or B(b)
            print(r)
            del r

    # TODO: The order of deletion does not exactly match, which we accept for
    # now.
    if True:
        print("Three arguments, A or B or C:")
        for a in range(2):
            for b in range(2):
                for c in range(2):
                    print("Case %d or %d or %d" % (a, b, c))
                    r = A(a) or B(b) or C(c)
                    print(r)
                    del r

    print("Two arguments, A and B:")
    for a in range(2):
        for b in range(2):
            print("Case %d and %d" % (a, b))
            r = A(a) and B(b)
            print(r)
            del r

    # See above
    if True:
        print("Three arguments, A and B and C:")
        for a in range(2):
            for b in range(2):
                for c in range(2):
                    print("Case %d and %d and %d" % (a, b, c))
                    r = A(a) and B(b) and C(c)
                    print(r)
                    del r


def comparisonChainOrderCheck():
    print("Checking order of comparison chains:")

    class A(int):
        def __init__(self, value):
            self.value = value

        def __del__(self):
            print("Doing del of %s" % self)

        def __le__(self, other):
            print("Test of %s <= %s" % (self, other))
            return self.value <= other.value

        def __str__(self):
            return "<%s %r>" % (self.__class__.__name__, self.value)

    class B(A):
        pass

    class C(A):
        pass

    # See above, not really doing it right.
    if False:
        print("Three arguments, A <= B <= C:")
        for a in range(3):
            for b in range(3):
                for c in range(3):
                    print("Case %d <= %d <= %d" % (a, b, c))
                    r = A(a) <= B(b) <= C(c)
                    print(r)
                    del r


dictOrderCheck()
separator()
listOrderCheck()
separator()
subscriptOrderCheck()
separator()
attributeOrderCheck()
separator()
operatorOrderCheck()
separator()
compareOrderCheck()
separator()
sliceOrderCheck()
separator()
generatorOrderCheck()
separator()
classOrderCheck()
separator()
inOrderCheck()
separator()
unpackOrderCheck()
separator()
superOrderCheck()
separator()
isinstanceOrderCheck()
separator()
rangeOrderCheck()
separator()
importOrderCheck()
separator()
hasattrOrderCheck()
separator()
getattrOrderCheck()
separator()
typeOrderCheck()
separator()
iterOrderCheck()
separator()
openOrderCheck()
separator()
unicodeOrderCheck()
separator()
nextOrderCheck()
separator()
longOrderCheck()
separator()
intOrderCheck()
separator()
callOrderCheck()
separator()
boolOrderCheck()
separator()
comparisonChainOrderCheck()

#     Python tests originally created or extracted from other peoples work. The
#     parts were too small to be protected.
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.



#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file


""" This module deals with finding and information about static libraries.

"""

import os

from nuitka.containers.OrderedSets import OrderedSet
from nuitka.PythonFlavors import (
    isAnacondaPython,
    isDebianPackagePython,
    isNuitkaPython,
)
from nuitka.PythonVersions import (
    getPythonABI,
    getSystemPrefixPath,
    python_version,
    python_version_str,
)
from nuitka.Tracing import general

from .FileOperations import getFileContentByLine, getFileList
from .Utils import getLinuxDistribution, isDebianBasedLinux, isWin32Windows

_ldconf_paths = None

_static_lib_cache = {}


def locateStaticLinkLibrary(dll_name):
    if dll_name not in _static_lib_cache:
        _static_lib_cache[dll_name] = _locateStaticLinkLibrary(dll_name)

    return _static_lib_cache[dll_name]


def _locateStaticLinkLibrary(dll_name):
    # singleton, pylint: disable=global-statement
    #
    global _ldconf_paths

    if _ldconf_paths is None:
        _ldconf_paths = OrderedSet()

        for conf_filemame in getFileList("/etc/ld.so.conf.d", only_suffixes=".conf"):
            for conf_line in getFileContentByLine(conf_filemame):
                conf_line = conf_line.split("#", 1)[0]
                conf_line = conf_line.strip()

                if os.path.exists(conf_line):
                    _ldconf_paths.add(conf_line)

    for ld_config_path in _ldconf_paths:
        candidate = os.path.join(ld_config_path, "lib%s.a" % dll_name)

        if os.path.exists(candidate):
            return candidate

    return None


_static_lib_python_path = False


def isDebianSuitableForStaticLinking():
    dist_name, _base, dist_version = getLinuxDistribution()

    if dist_name == "Debian":
        if dist_version is None:
            return True

        try:
            dist_version = tuple(int(x) for x in dist_version.split("."))
        except ValueError:
            # dist_version contains a non-numeric string such as "sid".
            return True

        return dist_version >= (10,)
    else:
        # TODO: Needs implementing potentially, Mint etc. are based
        # on something that should be considered.
        return True


def _getSysConfigVarLIBPL():
    # Return the LIBPL config variable, or None if it's not set or retrievable
    try:
        import sysconfig

        return sysconfig.get_config_var("LIBPL")

    except ImportError:
        # Cannot detect this properly for Python 2.6, but we don't care much
        # about that anyway.
        return None


def _getSystemStaticLibPythonPath():
    # Return driven function with many cases, pylint: disable=too-many-branches,too-many-return-statements

    sys_prefix = getSystemPrefixPath()
    python_abi_version = python_version_str + getPythonABI()

    if isNuitkaPython():
        # Nuitka Python has this.
        if isWin32Windows():
            return os.path.join(
                sys_prefix,
                "libs",
                "python" + python_abi_version.replace(".", "") + ".lib",
            )
        else:
            return os.path.join(
                sys_prefix,
                "lib",
                "libpython" + python_abi_version + ".a",
            )

    if isWin32Windows():
        # The gcc used on Windows for Anaconda is far too old for winlibs gcc
        # to use its library.
        if isAnacondaPython():
            return None

        candidates = [
            # Anaconda has this.
            os.path.join(
                sys_prefix,
                "libs",
                "libpython" + python_abi_version.replace(".", "") + ".dll.a",
            ),
            # MSYS2 mingw64 Python has this.
            os.path.join(
                sys_prefix,
                "lib",
                "libpython" + python_abi_version + ".dll.a",
            ),
        ]

        for candidate in candidates:
            if os.path.exists(candidate):
                return candidate
    else:
        candidate = os.path.join(
            sys_prefix, "lib", "libpython" + python_abi_version + ".a"
        )

        if os.path.exists(candidate):
            return candidate

        # For Python2 this works. TODO: Figure out Debian and Python3.
        if (
            python_version < 0x300
            and isDebianPackagePython()
            and isDebianSuitableForStaticLinking()
        ):
            candidate = locateStaticLinkLibrary("python" + python_abi_version)
        else:
            candidate = None

        if candidate is not None and os.path.exists(candidate):
            # Also check libz, can be missing
            if not locateStaticLinkLibrary("z"):
                general.warning(
                    "Error, missing 'libz-dev' installation needed for static lib-python."
                )

            return candidate

        # This is not necessarily only for Python3 on Debian, but maybe others as well,
        # but that's what's been tested.
        if python_version >= 0x300 and isDebianPackagePython() and isDebianBasedLinux():
            candidate = os.path.join(
                _getSysConfigVarLIBPL(),
                "libpython" + python_abi_version + "-pic.a",
            )

            if os.path.exists(candidate):
                return candidate

        libpl = _getSysConfigVarLIBPL()
        if libpl is not None:
            candidate = os.path.join(
                libpl,
                "libpython" + python_abi_version + ".a",
            )

            if os.path.exists(candidate):
                return candidate

    return None


def getSystemStaticLibPythonPath():
    global _static_lib_python_path  # singleton, pylint: disable=global-statement

    if _static_lib_python_path is False:
        _static_lib_python_path = _getSystemStaticLibPythonPath()

    return _static_lib_python_path


#     Part of "Nuitka", an optimizing Python compiler that is compatible and
#     integrates with CPython, but also works on its own.
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
